package com.example.mathseduc

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import androidx.appcompat.app.AppCompatActivity
import com.example.mathseduc.controllers.ControllerChapter
import com.example.mathseduc.controllers.ControllerLobby
import com.example.mathseduc.controllers.ControllerUtiliser
import com.example.mathseduc.models.Lobby
import okhttp3.MultipartBody

class MultiActivity : AppCompatActivity() {

    private lateinit var lobbyAdapter: LobbyAdapter
    private val handler = Handler(Looper.getMainLooper())
    private val refreshInterval: Long = 3000 // Refresh every 3 seconds

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_multi)

        var btnAddLobby = findViewById<Button>(R.id.btnAddLobby)

        val serverList = ControllerLobby.getLobbies()

        if (serverList != null) {

            val listView = findViewById<ListView>(R.id.listView)

            lobbyAdapter = LobbyAdapter(this, serverList)
            listView.adapter = lobbyAdapter

            listView.setOnItemClickListener { _, _, position, _ ->
                if (ControllerLobby.getNbPlayerInLobby(serverList[position].id) < serverList[position].nbplayers) {
                    val formDataBuilder = MultipartBody.Builder().setType(MultipartBody.FORM)
                    formDataBuilder.addFormDataPart("idplayer", MainActivity.idPlayerConnected.toString())
                    formDataBuilder.addFormDataPart("idlobby", serverList[position].id.toString())
                    formDataBuilder.addFormDataPart("playertime", "0")

                    ControllerUtiliser.createUtiliserByIdLobby(formDataBuilder)

                    val intent = Intent(this, ServerDetailsActivity::class.java)
                    intent.putExtra("serverName", serverList[position].name)
                    intent.putExtra("lobbyId", serverList[position].id)
                    startActivity(intent)
                } else {
                    Toast.makeText(this, "Oh nan, le serveur est déjà plein ! Réessayer plus tard.", Toast.LENGTH_SHORT).show()
                }
            }

            // Schedule periodic refresh using Handler and Runnable
            handler.postDelayed(refreshRunnable, refreshInterval)

        } else {
            Log.e("MultiActivity", "Error fetching server list")
        }

        btnAddLobby.setOnClickListener {
            val intent = Intent(this, CreateLobbyActivity::class.java)
            startActivity(intent)
        }

        val btnReturn = findViewById<ImageButton>(R.id.btnReturn)

        btnReturn.setOnClickListener {
            //val intent = Intent(this, MainActivity::class.java)
            //startActivity(intent)
            finish()
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        // Remove callbacks to prevent memory leaks
        handler.removeCallbacks(refreshRunnable)
    }

    private val refreshRunnable: Runnable = object : Runnable {
        override fun run() {
            // Refresh the server list
            val serverList = ControllerLobby.getLobbies()
            if (serverList != null) {
                lobbyAdapter.clear()
                lobbyAdapter.addAll(serverList)
                lobbyAdapter.notifyDataSetChanged()
            }
            // Schedule the next refresh
            handler.postDelayed(this, refreshInterval)
        }
    }

    class LobbyAdapter(context: Context, lobbies: ArrayList<Lobby>) :
        ArrayAdapter<Lobby>(context, 0, lobbies), View.OnClickListener {

        private val mContext: Context
        private val mLobbies: ArrayList<Lobby>

        private var selectedItem = -1

        // Get Access to Context and Data Array
        init {
            mContext = context
            mLobbies = lobbies
        }

        override fun getCount(): Int {
            return mLobbies.size
        }

        override fun getItemId(position: Int): Long {
            return mLobbies[position].id.toLong()
        }

        override fun getItem(position: Int): Lobby {
            return mLobbies[position]
        }

        override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
            var currentLobbyView = convertView
            if (currentLobbyView == null) {
                currentLobbyView =
                    LayoutInflater.from(context).inflate(R.layout.list_view_lobby, parent, false)
            }

            val name = currentLobbyView!!.findViewById<TextView>(R.id.name)
            val nbplayers = currentLobbyView!!.findViewById<TextView>(R.id.nbplayers)
            val idchapter = currentLobbyView!!.findViewById<TextView>(R.id.idchapter)
            val difficulty = currentLobbyView!!.findViewById<TextView>(R.id.difficulty)

            name.text = mLobbies[position].name
            nbplayers.text = ControllerLobby.getNbPlayerInLobby(mLobbies[position].id).toString() + "/" + mLobbies[position].nbplayers.toString()
            idchapter.text = ControllerChapter.getChapterNameById(mLobbies[position].idchapter)
            difficulty.text = mLobbies[position].difficulty.toString()

            return currentLobbyView
        }

        fun setSelectedItem(position: Int) {
            selectedItem = position
            notifyDataSetChanged()
        }

        fun getSelectedItem(): Lobby? {
            if (selectedItem != -1)
                return mLobbies[selectedItem]
            else
                return null
        }

        override fun onClick(p0: View?) {
        }
    }
}
